Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func ReaderWithContext ¶
ReaderWithContext arranges for stdin reads to observe context cancellation. When the underlying reader is closable, cancellation also closes it so blocked reads can unblock promptly.
func ScannerTokenLimit ¶
func WrapRedirectedFile ¶
Types ¶
type RedirectMetadata ¶
Click to show internal directories.
Click to hide internal directories.