Documentation
¶
Overview ¶
Package network provides the sandboxed HTTP client used by gbash.
This is a supported public extension package for callers that need to customize the HTTP transport, resolver, or request policy used by curl inside the sandbox. Most embedders should prefer the root `github.com/ewhauser/gbash` package and configure network access through its higher-level options.
Index ¶
- func IsDenied(err error) bool
- type AccessDeniedError
- type Client
- type Config
- type HTTPClient
- type HTTPDoer
- type InvalidConfigError
- type Method
- type MethodNotAllowedError
- type Option
- type RedirectNotAllowedError
- type Request
- type Resolver
- type Response
- type ResponseTooLargeError
- type TooManyRedirectsError
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type AccessDeniedError ¶
func (*AccessDeniedError) Error ¶
func (e *AccessDeniedError) Error() string
type HTTPClient ¶
type HTTPClient struct {
// contains filtered or unexported fields
}
type InvalidConfigError ¶
type InvalidConfigError struct {
Problems []string
}
func (*InvalidConfigError) Error ¶
func (e *InvalidConfigError) Error() string
type MethodNotAllowedError ¶
type MethodNotAllowedError struct {
Method string
}
func (*MethodNotAllowedError) Error ¶
func (e *MethodNotAllowedError) Error() string
type RedirectNotAllowedError ¶
type RedirectNotAllowedError struct {
URL string
}
func (*RedirectNotAllowedError) Error ¶
func (e *RedirectNotAllowedError) Error() string
type ResponseTooLargeError ¶
type ResponseTooLargeError struct {
MaxBytes int64
}
func (*ResponseTooLargeError) Error ¶
func (e *ResponseTooLargeError) Error() string
type TooManyRedirectsError ¶
type TooManyRedirectsError struct {
MaxRedirects int
}
func (*TooManyRedirectsError) Error ¶
func (e *TooManyRedirectsError) Error() string
Click to show internal directories.
Click to hide internal directories.